<?php

namespace App\Service;

error_reporting(E_ERROR | E_PARSE);

use App\Exception as Exception;

class MenuService {

    private $priceService;
    private $branchFactory;
    private $storeFactory;
    private $menuFactory;
    
    public function __construct (\App\Service\PriceService $priceService,
                                 \App\Factory\BranchFactory $branchFactory,
                                 \App\Factory\StoreFactory $storeFactory,
                                 \App\Factory\MenuFactory $menuFactory) {

        $this->priceService  = $priceService;
        $this->branchFactory = $branchFactory;
        $this->storeFactory  = $storeFactory;
        $this->menuFactory   = $menuFactory;
    }
    
    
    public function getBranchs() {
    	$branchsData = $this->branchFactory->getBranchsNames();
    	$branchs = [];

    	foreach($branchsData as $branch) {
    		
    		$standardStore  = $this->storeFactory->getDefaultStoreByBranch($branch['CDFILIAL']);

    		$storesData = $this->storeFactory->getStoresNames($branch['CDFILIAL']);
    		$stores 	= [];
			
			if(!count($storesData)){
				continue;
			}
			
    		foreach($storesData as $store) {
    			array_push($stores, [
    				'storeId'	=> $store['CDLOJA'],
    				'storeName' => $store['NMLOJA']
    			]);
    		}

			array_push($branchs, [
				'branchId'	 => $branch['CDFILIAL'],
				'branchName' => $branch['NMFILIAL'],
				'branchDefaultStore' => $standardStore,
				'stores'	 => $stores
			]);
    	}
		
		return $branchs;
    }
    
    public function getMenus() {
    	$branchs = $this->getBranchs();
    	
    	foreach($branchs as &$branch) {
    		
    		$defaultPOS = $this->storeFactory->getDefaultPOS($branch['branchId']);
			$branch['defaultPos'] = $defaultPOS['CDCAIXA'];
			
    		foreach($branch['stores'] as &$store) {
    			$pos	 = [];
				$posData = $this->storeFactory->getPOSNames($branch['branchId'], $store['storeId']);    			

				foreach($posData as $currentPos) {
					$menus = $this->getPosMenus($branch['branchId'], $currentPos['CDCAIXA']);
					if(empty($menus)){
						continue;
					}
					array_push($pos, [
						'posId' 	=> $currentPos['CDCAIXA'],
						'posName'	=> $currentPos['NMCAIXA'],
						'posType'   => $this->getPosType($currentPos['IDHABCAIXAVENDA']),
						'menus' => $menus
					]);
				}

	    		$store['pos'] = $pos;
    		}

	    	if(isset($store))
		    	unset($store);
    	}
    	
	  	if(isset($branch))
    		unset($branch);
    	
    	return $branchs;
    }

    private function getPosMenus($cdFilial, $cdCaixa) {
    	$menus = [];
    	$posMenusData = $this->storeFactory->getPosMenus($cdFilial, $cdCaixa);

    	foreach($posMenusData as &$menu) {
    		array_push($menus, [
    			'menuId'	=> $menu['NRCONFTELA'],
    			'validFrom' => $menu['DTINIVIGENCIA']
    		]);
    	}
    	
    	if(isset($menu))
    		unset($menu);
    		
   		return $menus;
    	
    }
    
    private function getPosType($type) {
    	$posType = "";
    	
    	switch($type) {
    		case 'POS':
				$posType = "Venda Balcão";
	    		break;

    		case 'CMD':
				$posType = "Comanda com Venda Balcão";
	    		break;

    		case 'RES':
				$posType = "Restaurante com Venda Balcão";
	    		break;

    		case 'FOS':
				$posType = "Restaurante/Comanda/Delivery com Venda Balcão";
	    		break;

    		case 'PKC':
				$posType = "Pocket Comanda";
	    		break;

    		case 'PKR':
				$posType = "Pocket Restaurante";
	    		break;

    		case 'TAA':
				$posType = "Terminal de Auto-Atendimento";
	    		break;

    		case 'EVB':
				$posType = "Delivery Venda Balcão";
	    		break;

    		case 'CTL':
				$posType = "Controle de Produção";
	    		break;

    		case 'CSE':
				$posType = "Controle de Saída/Entrega";
	    		break;

    		case 'FKB':
				$posType = "Microterminal";
	    		break;

    		case 'TOT':
				$posType = "Terminal de Consulta";
	    		break;

    		case 'APC':
				$posType = "Auto Pagamento Comanda";
	    		break;

    		case 'CPE':
				$posType = "Controle de Pedidos";
	    		break;

    		case 'TCO':
				$posType = "Terminal de Checkout de Comanda";
	    		break;

    		case 'CDE':
				$posType = "Call Center Delivery";
	    		break;

    		case 'SDE':
				$posType = "Site Delivery";
	    		break;

    		case 'CAT':
				$posType = "Catraca";
	    		break;


    		case 'APP':
				$posType = "Aplicativo Mobile";
	    		break;

    		case 'OGO':
				$posType = "OdhenGo - Inteligência Artificial";
	    		break;

			default:
				$posType = "Desconhecido";
            	break;
       	}

		return $posType;
    }
    
    public function getMenusUpdates() {
    	$updates = [];
    	$updatesData = $this->menuFactory->getMenusUpdates();

    	foreach($updatesData as &$update) {
    		array_push($updates, [
    			'date'          => $update['DTOPERACAOVND'],
    			'description'   => $update['DSOPERACAOVND'],
    			'branch'        => $update['CDFILLOGOPERVND']
    		]);
    	}
    	
    	if(isset($update))
    		unset($update);
    		
   		return $updates;
    	
    }
    
	public function getProducts($cdFilial, $nrConfTela) {
        
        /*get the CDFILIAL filed from the CONFTELACAIXA table. This field can be different from the branch received in the request because of mapping reasons.
        The branch received in the request will always be the CDFILICAIXA field.*/
        $cdFilial2 = $this->storeFactory->getConftelacaixaCdfilial($cdFilial, $nrConfTela);
    	
    	$products = $this->menuFactory->getGeneralProducts($cdFilial2, $nrConfTela);
    	
    	$blockedProducts = $this->menuFactory->getBlockedProducts($cdFilial);

    	$pricingTable = $this->priceService->getGeneralPricingTable($cdFilial);
        
    	$pricingTableValidity = $this->priceService->getGeneralPricingTableValidity($pricingTable);
        
        $customPrice     = [];
    	$prices = $this->priceService->getGeneralPrices($pricingTable, $pricingTableValidity);

    	foreach($products as &$product){
    		$product['FULLPRICE'] = isset($prices[$product['CDPRODUTO']]['VRPRECITEM']) ? $prices[$product['CDPRODUTO']]['VRPRECITEM'] : '0,0';
    	}
    	
    	if(isset($product)){
    		unset($product);
    	}

		$filteredProducts = [];

		foreach($products as $product) {
			//validar preço
			$price = $this->tofloat($product['FULLPRICE']);
			
			if($price < 0.01){
				continue;
			}
			
			$productExists = false;
			foreach($filteredProducts as $filteredProduct){
				if($filteredProduct['itemId'] == $product['CDPRODUTO']){
					$productExists = true;
					break;
				}
			}
			
			if($productExists){
				continue;
			}
			
			array_push($filteredProducts, [
				'itemId'	=> $product['CDPRODUTO'],
				'itemName'	=> $product['NMPRODUTO'],
				'itemValue' => $price
			]);
		}
		
		$formattedBlockedProducts = array();
		
		foreach($blockedProducts as $blockedProduct){
			array_push($formattedBlockedProducts, $blockedProduct['CDPRODUTO']);
		}

		return array(
			'branchId' => $cdFilial,
			'menuId' => $nrConfTela,
			'products' => $filteredProducts,
			'blockedProducts' => $formattedBlockedProducts
		);
	}


    public function loadMenu($CDFILIAL) {
		$currentDay   = date("w") + 1;
    	$currentDate  = date("d/m/y");
    	
		$CDCLIENTE  = $this->branchFactory->getStandardClient($CDFILIAL)["CDCLIENTE"];
		$CDVENDEDOR = $this->storeFactory->getStandardVendor($CDFILIAL);
		$CDCAIXA    = $this->storeFactory->getStandardCashier($CDFILIAL, $CDVENDEDOR)["CDCAIXA"];
		$CDLOJA     = $this->storeFactory->getStandardStore($CDFILIAL, $CDVENDEDOR);
		
    	$CDTIPOCONS = "T";
        $NRDIASEMANABLOQ = date("w") + 1;
    	
    	$products        = $this->menuFactory->getProducts($CDFILIAL, $CDCAIXA, $NRDIASEMANABLOQ);
        $comboProducts   = $this->menuFactory->getComboProducts($CDFILIAL, $CDCAIXA, $CDLOJA);
    	$pricingTable    = $this->priceService->getPricingTable($CDCLIENTE, $CDFILIAL, $CDLOJA);
    	$effectiveDate   = $this->priceService->checkEffective($pricingTable["CDFILTABPREC"], $pricingTable["CDTABEPREC"]);
    	$isHoliday       = $this->priceService->isHoliday($CDFILIAL, $currentDate);
    	$discountType	 = $this->priceService->getProductDiscountType($CDFILIAL, $CDTIPOCONS);
        
        $customPrice     = [];
    	if($isHoliday)
    		$customPrice = $this->priceService->getPricesOnHoliday($CDFILIAL, $pricingTable["CDTABEPREC"], $effectiveDate, $currentDay);
    
 		$customPrice = $this->priceService->getPricesByDay($CDFILIAL, $pricingTable["CDTABEPREC"], $effectiveDate, $currentDay, $CDTIPOCONS);
    	
    	$prices = $this->priceService->getPrices($CDFILIAL, $pricingTable["CDFILTABPREC"], $pricingTable["CDTABEPREC"], $effectiveDate, $NRDIASEMANABLOQ);
    
    	//Calculates discount
    	$discounts = [];
    	if (!empty($customPrice)) {
    		foreach($customPrice as $dayPrice) {
    			if(array_key_exists($dayPrice['CDPRODUTO'], $prices)) {
    				if($dayPrice['IDPERVALORPR'] == 'V') { //Value
    					if($dayPrice['IDDESCACREPR'] == 'D') {
    						//Discount by value
    						$prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] = $prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] - $dayPrice['VRPRECODIA'];
							
							array_push($discounts, [
								'CDPRODUTO'    => $dayPrice['CDPRODUTO'],
								'IDPERVALORPR' => $dayPrice['IDPERVALORPR'],
								'IDDESCACREPR' => $dayPrice['IDDESCACREPR'],
								'DISCOUNT'	   => $dayPrice['VRPRECODIA']
							]);
    					} else {
    						//Increase by value
    						$prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] = $prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] + $dayPrice['VRPRECODIA'];
    					}

    				} else { //Percentage
    					$percent = $dayPrice['VRPRECODIA'] / 100;
    					if($dayPrice['IDDESCACREPR'] == 'D') {
    						//Discout by percentage
    						$discountPercentage = $prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] * (1 - $percent);
    						$prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] = $discountPercentage;
							
							array_push($discounts, [
								'CDPRODUTO'    => $dayPrice['CDPRODUTO'],
								'IDPERVALORPR' => $dayPrice['IDPERVALORPR'],
								'IDDESCACREPR' => $dayPrice['IDDESCACREPR'],
								'DISCOUNT'	   => $discountPercentage
							]);
    					} else {
    						//Increase by percentage
    						$prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] = $prices[$dayPrice['CDPRODUTO']]['VRPRECITEM'] * (1 + $percent);
    					}
    				}
    			}
    		}
    	}
    
    	/* Sets the prices and other stuffs. */
    	$pricedProducts = [];
    	
    	foreach ($products as $product) {
    		try {
    			if(empty($product['IMGGRUPO'])) {
                	$product['IMGGRUPO']      = '';
                	$product['hasGroupImage'] = false;
    			} else {
    				$product['IMGGRUPO']      = $product['IMGGRUPO'];
    				$product['hasGroupImage'] = true;
    			}
                
                $productDiscountType = array_filter($discountType, function($prod) use ($product) {
                	return $prod["CDPRODUTO"] === $product["CDPRODUTO"];
                });
    
                $product['IMGPRODUCT'] = empty($product['IMGPRODUCT']) ? '' : $product['IMGPRODUCT'];
                $product['PROMOPRODS'] = []; // Declares array to be used by the front end to be filled with SmartPromo products.
                $product['COMBOPRODS'] = []; // Declares array to be used by the front end to be filled with Combo Products.
                $product['SELECTION']  = []; // Declares array to be used by the front end to be filled with selections (currently only combo products).
                $product['TIPO']	   = array_shift($productDiscountType)["TIPO"];
    
    			// IDIMPPRODUTO defines if the product will be printed or not.
    			// Products with IDIMPPRODUTO set different than 1 are often SmartPromo parent products whose overall price will be defined by their children.
    			if ($product['IDIMPPRODUTO'] == '' || $product['IDIMPPRODUTO'] == '1') {
    				$product['PRICE'] = floatval(number_format($prices[$product['CDPRODUTO']]['VRPRECITEM'], 2, '.', ','));
    				$absPrice = floatval(number_format($prices[$product['CDPRODUTO']]['FULLPRICE'], 2, '.', ','));
    				$product['VRDESC'] = $absPrice - $product['PRICE'];
    				$product['FULLPRICE'] = $absPrice;
    			} else {
    				$product['FULLPRICE'] = floatval(number_format(0, 2, '.', ','));
    				$product['PRICE'] = floatval(number_format(0, 2, '.', ','));
    			}
    			
    			array_push($pricedProducts, $product);
    		} catch (\Exception $e) {
    			// If something goes wrong, we don't add the product. Such is the way of life. u__u
    		}
    	}
        /* Prices the combo products. */
        $pricedCombos = [];
        
        foreach ($comboProducts as $comboGroup) {
            try {
                foreach ($comboGroup['COMBOPRODS'] as &$comboItem) {
                    try {
                    	$comboItem['FULLPRICE'] = floatval(number_format($prices[$comboItem['CDIDENTBUTON']]['FULLPRICE'], 2, '.', ','));
                    	$comboItem['PRICE'] = floatval(number_format($prices[$comboItem['CDIDENTBUTON']]['VRPRECITEM'], 2, '.', ','));
                    } catch (\Exception $e) {
                        $comboItem['FULLPRICE'] = floatval(0);
                        $comboItem['PRICE'] = floatval(0);
                    }
                }
                
                array_push($pricedCombos, $comboGroup);
            } catch (\Exception $e) {
                // If something goes wrong, we don't add the product here either. It has been decided.
            }
        }
    
        /* Get observations. They are sent in a separate datasource. */
        $observations = $this->menuFactory->getObservations($CDFILIAL, $CDLOJA, $NRDIASEMANABLOQ);
        
        $pricedObservations = [];
        
    	foreach ($observations as $observation) {
			
    		try {
    			if($observation['IDCONTROLAOBS'] === 'A' && $observation['CODIGOOCORRENCIA'] !== null && isset($prices[$observation['CODIGOOCORRENCIA']])) {
    				$observation['PRICE'] = floatval(number_format($prices[$observation['CODIGOOCORRENCIA']]['VRPRECITEM'], 2, '.', ','));
	    			$observation['FULLPRICE'] = $observation['PRICE'];
    				
    			} else {
    				$observation['PRICE'] = floatval(number_format(0, 2, '.', ','));
    				$observation['FULLPRICE'] = floatval(0);
    			}

    			//Retira o preço de observação do tipo 'R' (Retirada) 
    			if($observation['IDCONTROLAOBS'] == 'R' && $observation['IDCONTROLAOBS'] == 'O') {
    				$observation['CODIGOOCORRENCIA'] = null;
					if(isset($observation['PRICE'])) {
	    				unset($observation['PRICE']);
					}
					
					if(isset($observation['FULLPRICE'])) {
	    				unset($observation['FULLPRICE']);
					}
    			}

    			array_push($pricedObservations, $observation);
    		} catch (\Exception $e) {
    			// If something goes wrong, we don't add the product. Such is the way of life. u__u
    		}
        }
        
        $obsNames = $this->extractDsOcorr($observations);
        $pricedObservations = $this->organizeObservationsStructure($pricedObservations, $obsNames);
        $organizedProducts = $this->organizeProductsStructure($pricedProducts, $pricingTable);
        $organizedCombos = $this->organizeComboProducts($pricedCombos);
        
        $data = [
            "prices"		=> $prices,
            "products"		=> array_merge($organizedProducts, $organizedCombos),
            "observations"  => $pricedObservations,
            "obsNames"		=> $obsNames,
            "discounts" 	=> $discounts
        ];
    	
        return $this->assembleMenuRepository($data);
    }
    
    private function assembleMenuRepository($data) {
		$menuFactory["Products"]     = [];
		$menuFactory["Observations"] = [];
		
		foreach ($data["products"] as $group) {
			if (isset($group["PRODUCTS"])) {
				foreach ($group["PRODUCTS"] as $product) {
					try {
						$CDPRODUTO = $product["CDPRODUTO"];
						
						// Prices
						if (isset($data["discounts"][$CDPRODUTO]))
							$product["PRICE"] = floatval(number_format(($data["prices"][$CDPRODUTO] - $data["discounts"][$CDPRODUTO]), 2, ".", ","));
						else
							$product["PRICE"] = $data["prices"][$CDPRODUTO];
					} catch (\Exception $e) {
						
					}
				}
			} else if (isset($group["COMBOPRODS"])) {
				foreach ($group["COMBOPRODS"] as $subGroup) {
					foreach($subGroup["COMBOPRODS"] as $comboItem) {
						try {
							$CDIDENTBUTON = $comboItem["CDIDENTBUTON"];
							
							if (isset($data["discounts"][$CDIDENTBUTON]))
								$comboItem["PRICE"] = floatval(number_format(($data["prices"][$CDIDENTBUTON] - $data["discounts"][$CDIDENTBUTON]), 2, ".", ","));
							else
								$comboItem["PRICE"] = $data["prices"][$CDIDENTBUTON];
						} catch (\Exception $e) {
							
						}
					}
				}
			}
			
			array_push($menuFactory["Products"], $group);
		}
		
		array_push($menuFactory["Observations"], $data["observations"]);
		
		return $menuFactory;
	}
    
    private function organizeProductsStructure($products, $pricingTable) {
    	$organizedProducts = [];
    	
    	foreach ($products as &$product) {
    		if ($product["IDTIPOCOMPPROD"] === "1") {
    			$product["IDTIPOCOMPPROD"] = "0";
    		}
    		if ($product["IDTIPOCOMPPROD"] === "3") {
    			$product["COMBOPRODS"] = $this->getGrupPromo($product["CDPRODUTO"]);
    			
    			foreach ($product["COMBOPRODS"] as &$category) {
    				$category["PRODUCTS"] = $this->loadProdGrup($category["CDPRODPROMOCAO"], $category["CDGRUPROMOC"], $product["CDFILIAL"], $pricingTable["CDTABEPREC"], $product["CDLOJA"]);
    				
    				$this->takeAndUnsetIndex($category, "CDPRODPROMOCAO");
    				$this->takeAndUnsetIndex($category, "NRORDPROMOGRUP");
    				
    				foreach ($category["PRODUCTS"] as &$possibleProduct) {
    					$this->takeAndUnsetIndex($possibleProduct, "PRODUTO_PAI_PROMO");
    					$this->takeAndUnsetIndex($possibleProduct, "GRUPO_PROMO");
    					$this->takeAndUnsetIndex($possibleProduct, "CDFILIAL");
    					$this->takeAndUnsetIndex($possibleProduct, "NMGRUPROMOC");
    					
    					$descVal = empty($possibleProduct["VRDESC"]) ? 0 : $possibleProduct["VRDESC"];
    				}
    			}
    			
    		}

    		$index = $this->takeAndUnsetIndex($product, "CDGRUPO");
    		
    		$this->takeAndUnsetIndex($product, "CDFILIAL");
    		$this->takeAndUnsetIndex($product, "CDLOJA");
    		$this->takeAndUnsetIndex($product, "hasGroupImage");
    		
    		if (empty($organizedProducts[$index])) {
    			$NMGRUPO    = $this->takeAndUnsetIndex($product, "NMGRUPO");
    			$IMGGRUPO   = $this->takeAndUnsetIndex($product, "IMGGRUPO");
    			$NRCONFTELA = $this->takeAndUnsetIndex($product, "NRCONFTELA");
    			
    			$organizedProducts[$index] = [
					"groupName" => $NMGRUPO,
					"products"  => [$product]
				];
    		} else {
    			// Since the group already exists, there"s no reason to rewrite it"s properties, 
    			// therefore we ignore the return of indexes whom are being unsetted
    			$this->takeAndUnsetIndex($product, "NMGRUPO");
    			$this->takeAndUnsetIndex($product, "NRCONFTELA");
    			unset($product["IMGGRUPO"]);
    			
				array_push($organizedProducts[$index]["products"], $product);
    		}
    	}
    	foreach ($organizedProducts as &$product) {
    		if (isset($product["products"]))
    			unset($product["products"][0]["IMGGRUPO"]);
    			
			$this->templateProducts($product);
    	}
    	
    	if(isset($product)) unset($product);
    	
    	ksort($organizedProducts);
    	return $organizedProducts;
    }
    
    private function organizeComboProducts($comboProducts) {
		$organizedCombos = [];
		foreach ($comboProducts as $combo) {
			$index = $this->takeAndUnsetIndex($combo, "CDGRUPO");
			
			foreach ($combo["COMBOPRODS"] as &$comboItem) {
				$this->takeAndUnsetIndex($comboItem, "CDFILIAL");
				$this->takeAndUnsetIndex($comboItem, "CDLOJA");
				$this->takeAndUnsetIndex($comboItem, "CDARVPROD");
				$this->takeAndUnsetIndex($comboItem, "IMGPRODUCT");
				$this->takeAndUnsetIndex($comboItem, "NOIMAGE");
			}
			
			if (empty($organizedCombos[$index])) {
				$NMGRUPO  = $this->takeAndUnsetIndex($combo, "NMGRUPO");
				$IMGGRUPO = $this->takeAndUnsetIndex($combo, "IMGGRUPO");
				
				$this->takeAndUnsetIndex($combo, "CDFILIAL");
				$this->takeAndUnsetIndex($combo, "CDLOJA");
				
				$organizedCombos[$index] = [
					"groupName"     => $NMGRUPO,
					"pizzaProducts" => [$combo]
				];
    		} else {
				// Since the group already exists, there's no reason to rewrite it's properties, 
				// therefore we ignore the return of indexes whom are being unsetted
				$this->takeAndUnsetIndex($combo, "NMGRUPO");
				$this->takeAndUnsetIndex($combo, "IMGGRUPO");
				$this->takeAndUnsetIndex($combo, "CDFILIAL");
				$this->takeAndUnsetIndex($combo, "CDLOJA");
				array_push($organizedCombos[$index]["pizzaProducts"], $combo);
			}
			
			$organizedCombos[$index]["pizzaProducts"] = $this->templatePizza($organizedCombos[$index]);
		}
		
		return $organizedCombos;
    }
    
    private function organizeObservationsStructure(&$observations, $names) {
    	try {
    		$organizedObs = [];
	    	
	    	foreach ($names as $obs)
	    		$organizedObs[$obs["CDGRPOCOR"]] = [];
	    		
	    	foreach ($observations as &$observation) {
				$CDGRPOCOR     = $this->takeAndUnsetIndex($observation, "CDGRPOCOR");
				$CDOCORR       = $this->takeAndUnsetIndex($observation, "CDOCORR");
				$IDCONTROLAOBS = $this->takeAndUnsetIndex($observation, "IDCONTROLAOBS");
				$CDOCORRENCIA  = $this->takeAndUnsetIndex($observation, "CODIGOOCORRENCIA");
				$CDPRODUTO     = $this->takeAndUnsetIndex($observation, "CDPRODUTO");
				$PRICE         = $this->takeAndUnsetIndex($observation, "PRICE");
				$FULLPRICE     = $this->takeAndUnsetIndex($observation, "FULLPRICE");
				
	    		if (empty($organizedObs[$CDGRPOCOR])) {
	    			$organizedObs[$CDGRPOCOR] = [
						$CDOCORR => $this->returnControlaObsStructure(
							$names, 
							$CDGRPOCOR, 
							$CDOCORR, 
							$CDOCORRENCIA, 
							$CDPRODUTO, 
							$PRICE, 
							$FULLPRICE, 
							$IDCONTROLAOBS
						)
					];
				} else if (empty($organizedObs[$CDGRPOCOR][$CDOCORR])) {
					$organizedObs[$CDGRPOCOR][$CDOCORR] = $this->returnControlaObsStructure(
						$names, 
						$CDGRPOCOR, 
						$CDOCORR, 
						$CDOCORRENCIA, 
						$CDPRODUTO, 
						$PRICE, 
						$FULLPRICE, 
						$IDCONTROLAOBS
					);
				} else {
					if ($IDCONTROLAOBS === "O") array_push(
						$organizedObs[$CDGRPOCOR][$CDOCORR]["O"]["products"], 
						$CDPRODUTO
					);
					
					else if ($IDCONTROLAOBS === "A") array_push(
						$organizedObs[$CDGRPOCOR][$CDOCORR]["A"]["products"], 
						$CDPRODUTO
					);
					
					else if ($IDCONTROLAOBS === "R") array_push(
						$organizedObs[$CDGRPOCOR][$CDOCORR]["R"]["products"], 
						$CDPRODUTO
					);
				}
			}

			return $organizedObs;
		} catch(\Exception $e){
			echo($e->getMessage());
		}
	}
    
    private function returnControlaObsStructure(
		$names, $CDGRPOCOR, 
		$CDOCORR, $CDOCORRENCIA, 
		$CDPRODUTO, $PRICE, 
		$FULLPRICE, $IDCONTROLAOBS
	) {

		$obsNameArr = array_filter(
			$names, 
			function ($obsName) use ($CDGRPOCOR, $CDOCORR, $CDOCORRENCIA) {
				return $obsName["CDGRPOCOR"]     == $CDGRPOCOR 
					&& $obsName["CDOCORR"]       == $CDOCORR 
					&& $obsName["CDOCORRENCIA"]  == $CDOCORRENCIA;
			}
		);
		
		$dsOcorr = array_shift($obsNameArr)["DSOCORR"];
		
		$o = $a = $r = []; // initialize observations
		
		if($IDCONTROLAOBS === 'A') {
			$a = [
				"observationName"  => $dsOcorr,
				"observationId"    => $CDOCORRENCIA,
				"observationValue" => $PRICE,
				"products"         => [$CDPRODUTO]
			];
    	} else if($IDCONTROLAOBS === 'O') {
    		$o = [
    			"observationName" => $dsOcorr,
				"products"  	  => [$CDPRODUTO]	
			];
    	} else {
    		$r = [
				"observationName" => $dsOcorr,
				"products"  	  => [$CDPRODUTO]		
			];
    	}
    	
    	return [
    		"O" => $o,
			"A" => $a,
			"R" => $r
    	];
    }

    private function getGrupPromo($CDPRODUTO) {
    	return $this->menuFactory->getGrupPromo($CDPRODUTO);
    }

    private function loadProdGrup(
		$CDPRODPROMOCAO, 
		$CDGRUPROMOC, 
		$CDFILIAL, 
		$CDTABEPREC, 
		$CDLOJA
    ) {
		$prodGrupo       = [];
	    $NRDIASEMANABLOQ = date("w") + 1;
	    $DTINIVGPREC	 = $this->priceService->checkEffective($CDFILIAL, $CDTABEPREC);
		$priceLessProd   = $this->menuFactory->getProdGrup($CDPRODPROMOCAO, $CDGRUPROMOC, $CDFILIAL, $CDTABEPREC, $CDLOJA, $NRDIASEMANABLOQ, $DTINIVGPREC);
		
		foreach ($priceLessProd as $item) {
			
			if (!isset($item["PRICE"]))
				throw new \Exception("Preço do item " . $item['NOME'] . " não parametrizado.");
			
			$item["PRICE"] = floatval(number_format($item["PRICE"], 2, ".", ","));
			$item["FULLPRICE"] = $item["PRICE"];
			
			if ($item["VRDESC"] != null){
				$item["VRDESC"] = floatval(number_format($item["VRDESC"], 2, ".", ","));

				if ($item["TIPO"] == "P") {
					$item["VRDESC"] = 1 - ($item["VRDESC"] / 100);
					$item["PRICE"]  = $item["PRICE"] * $item["VRDESC"];
				} else if ($item["TIPO"] == "V") {
					$item["PRICE"]  = $item["PRICE"] - $item["VRDESC"];
				}
			}

			if ($item["PRICE"] < 0.01) {
				$item["PRICE"] = 0.01;
			}
			
			array_push($prodGrupo, $item);
		}
		
		return $prodGrupo;
	}
	
	public function getPricingTables() {
    	$menusData = $this->menuFactory->getPricingTables();
    	$menus = [];

    	foreach($menusData as $menu) {
			$menuData = [
				'tableId'       => $menu['CDTABEPREC'],
				'tableName'     => $menu['NMTABEPREC'],
				'validFrom'     => $menu['DTINIVGPREC'],
				'validUntil'    => $menu['DTFINVGPREC'],
				'branches'      => []
			];
			
			$branchesData = $this->branchFactory->getMenuBranches($menu['CDTABEPREC']);
			foreach($branchesData as $branch){
				array_push($menuData['branches'], [
					'branchId'	 => $branch['CDFILIAL'],
					'branchName' => $branch['NMFILIAL']
				]);
			}
			
			if(isset($branch))
    			unset($branch);
    			
    		array_push($menus, $menuData);
    	}
    	
    	if(isset($menu))
    		unset($menu);
		
		return $menus;
    }
    
    public function getProductsByPricingTable($cdTabePrec) {
    	$pricingTableValidity = $this->priceService->getGeneralPricingTableValidity($cdTabePrec);
    	$products = $this->priceService->getGeneralPrices($cdTabePrec, $pricingTableValidity);
    	$NRDIASEMANPR = date("w") + 1;
    	$diffPrices = $this->priceService->getDiffPrices($cdTabePrec, $NRDIASEMANPR);

    	foreach($products as &$product){
    		if(isset($diffPrices[$product['CDPRODUTO']]['VRPRECODIA'])){
    			$product['FULLPRICE'] = $diffPrices[$product['CDPRODUTO']]['VRPRECODIA'];
    		} else if(isset($product['VRPRECITEM'])){
    			$product['FULLPRICE'] = $product['VRPRECITEM'];
    		} else{
    			$product['FULLPRICE'] = '0,0';
    		}
    	}
    	
    	if(isset($product)){
    		unset($product);
    	}

		$filteredProducts = [];
		foreach($products as $product) {
			//validar preço
			$price = $this->tofloat($product['FULLPRICE']);
			
			if($price < 0.01){
				continue;
			}
			
			$productExists = false;
			foreach($filteredProducts as $filteredProduct){
				if($filteredProduct['itemId'] == $product['CDPRODUTO']){
					$productExists = true;
					break;
				}
			}
			
			if($productExists){
				continue;
			}
			
			array_push($filteredProducts, [
				'itemId'	=> $product['CDPRODUTO'],
				'itemName'	=> $product['NMPRODUTO'],
				'itemValue' => $price
			]);
		}

		return array(
			'tableId' => $cdTabePrec,
			'products' => $filteredProducts
		);
	}
    
    private function takeAndUnsetIndex(&$array, $index) {
    	if (isset($array[$index])) {
	    	$indexValue = $array[$index];
	    	unset($array[$index]);
    		
    		return $indexValue;
    	}
    	
    	return false;
    }
    
    private function templatePizza ($organizedCombos) {
		$subItemsArr  = [];
		$pizzaProdArr = [];
		
		foreach($organizedCombos["pizzaProducts"] as $pzProd) {
			foreach($pzProd["COMBOPRODS"] as $cp) {
				$template = [
					"subItemName" => $cp["DSOCORR"],
					"subItemId"   => $cp["CDOCORR"],
					"productId"   => $cp["CDPRODUTO"],
					"description" => $cp["DESCRIPTION"],
					"value" 	  => $cp["PRICE"]
				];
				
				array_push($subItemsArr, $template);
			}
			
			array_push($pizzaProdArr, [
				"pizzaName" 	 => $pzProd["DSBUTTON"],
				"pizzaValue"	 => $pzProd["PRICE"],
				"pizzaLimit"	 => $pzProd["NRLIMPRODCOM"],
				"pizzaTaxMethod" => $pzProd["IDTIPCOBRA"],
				"products"  	 => $subItemsArr
			]);
		}
		
		return $pizzaProdArr; 
    }
    
    private function templateProducts (&$organizedProducts) {
		$productsArr   = [];
		
		foreach($organizedProducts["products"] as $product) {
			$comboItemsArr = [];
			
			if(!empty($product["COMBOPRODS"])) {
				foreach($product["COMBOPRODS"] as $comboProd) {
					$subProdArr = [];
					
					if(!empty($comboProd["PRODUCTS"])) {
						foreach($comboProd["PRODUCTS"] as $csp) {
							array_push($subProdArr, [
								"subItemName"		   => $csp["NOME"],
								"subItemId"			   => $csp["CDPRODUTO"],
								"subItemValue"		   => $csp["PRICE"],
								"subItemDiscountValue" => isset($csp["VRDESC"]) ?  $csp["VRDESC"] : 0,
								"subItemDiscountType"  => $csp["TIPO"]
							]);
						}
					}
					
					array_push($comboItemsArr, [
						"subProductGroupName"  => $comboProd["NMGRUPROMOC"],
						"subProductGroupId"	   => $comboProd["CDGRUPROMOC"],
						"subProductGroupLimit" => $comboProd["QTPRGRUPPROMOC"],
						"subProductGroupMin"   => $comboProd["QTPRGRUPROMIN"],
						"subProductGroupItems" => $subProdArr,
					]);
				}
			}
			
			$templateSubItems = [
				"itemName"		  => $product["DSBUTTON"],
				"itemId"		  => $product["CDPRODUTO"],
				"itemModifierId"  => $product["IDTIPOCOMPPROD"],
				"itemValue"		  => $product["PRICE"],
				"itemSubProducts" => $comboItemsArr 
			];
			
			array_push($productsArr, $templateSubItems); 
		}
		
		$organizedProducts["products"] = $productsArr;
	}
    
    private function extractDsOcorr(&$observations) {
    	$obsNames = [];
    	
    	foreach ($observations as &$obs) {
    		$dsOcorr = $obs["DSOCORR"] ? $obs["DSOCORR"] : false;
    		
    		if ($dsOcorr) {
    			array_push($obsNames, [
    				"CDGRPOCOR"    => $obs["CDGRPOCOR"],
    				"CDOCORR"      => $obs["CDOCORR"],
    				"CDOCORRENCIA" => $obs["CODIGOOCORRENCIA"],
    				"DSOCORR"      => $obs["DSOCORR"]
    			]);
    		}
    			
    		unset($obs["DSOCORR"]);
    	}
    	
    	return $obsNames;
    }
    
	private function tofloat($num) {
		$num = str_replace(',', '.', $num);
		$num = bcadd('0.00', $num, 2);
	    $dotPos = strrpos($num, '.');
	    $commaPos = strrpos($num, ',');
	    $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos :
	        ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
	  
	    if (!$sep) {
	        return floatval(preg_replace("/[^0-9]/", "", $num));
	    }
	
	    return floatval(
	        preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
	        preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
	    );
	}
}